function [grid, cfg] = prepare_dipole_grid(cfg, vol, sens);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION that helps to prepare a regular grid
% this is the used in sourceanalysis and dipolefitting
%
% Configuration options for grid definition
%   cfg.xgrid      = something like -20:1:20 or 'auto'
%   cfg.ygrid      = something like -20:1:20 or 'auto'
%   cfg.zgrid      = something like   0:1:20 or 'auto'
%   cfg.resolution = number (e.g. 1 cm) required for automatic grid generation
%
% Configuration options for cortex segmentation
%   cfg.mri           = can be filename, MRI structure or segmented volume structure
%   cfg.sourceunits   = 'mm' or 'cm' (default is 'cm')
%   cfg.mriunits      = 'mm' or 'cm' (default is 'mm')
%   cfg.threshold     = 0.1, relative to the maximum value in the segmentation
%   cfg.smooth        = 5, smoothing in voxels
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Copyright (C) 2004, Robert Oostenveld
%
% $Log: prepare_dipole_grid.m,v $
% Revision 1.17  2006/03/06 09:30:19  roboos
% improved the generation of a grid based on a segmented MRI, now also works in case the spanned box is outside the MRI
% some other small changes to the code
%
% Revision 1.16  2006/02/07 22:19:21  roboos
% changed a fprintf statement with feedback
%
% Revision 1.15  2006/01/24 13:59:37  roboos
% added piece of code to the end that makes the auto-generated grid tighter around the brain
%
% Revision 1.14  2005/12/14 14:18:18  roboos
% switched to using headsurface subfunction for determining inside/outside points
%
% Revision 1.13  2005/11/01 07:56:15  roboos
% made some error messages more consistent with each other, no functional changes
%
% Revision 1.12  2005/06/01 08:00:26  roboos
% added infinite medium for magnetic dipole, all dipoles are 'inside'
%
% Revision 1.11  2005/05/17 17:50:49  roboos
% changed all "if" occurences of & and | into && and ||
% this makes the code more compatible with Octave and also seems to be in closer correspondence with Matlab documentation on shortcircuited evaluation of sequential boolean constructs
%
% Revision 1.10  2005/04/25 16:10:47  roboos
% implemented flexible scaling of source and mri (cfg.sourceunits/cfg.mriunits)
% changed handling of cfg.mri to be more general
% renamed some of the local variable names in the cortex segmentation
% replaced intersect() call with a smarter way of determining source grid locations in the cortex segmentation
% the call to SEGMENTVOLUME for the smoothing was broken, replaced with a local call to SPM2 spm_conv function
%
% Revision 1.9  2004/10/27 16:08:15  roboos
% added grid.subspace as one of the recognized fields that is kept
%
% Revision 1.8  2004/10/14 07:11:56  jansch
% changed outputted grid-positions to head-coordinates. added hidden option
% for setting a threshold to do inside-outside detection on the segmented volume
%
% Revision 1.7  2004/09/22 10:42:20  roboos
% added 'auto' defaults for x/y/zgrid
% convert brain segmentation to double before performing computations
% if the precomputed grid contains an lbex field, keep it
%
% Revision 1.6  2004/09/17 11:21:45  jansch
% inserted the possibility to create a grid, based on the cortex-segmentation
%
% Revision 1.5  2004/08/23 10:21:37  roboos
% added precomputed filters from grid AND from grid.avg
%
% Revision 1.4  2004/08/19 12:27:32  roboos
% fixed bug in incorrect copying of xgrid/ygrid/zgrid
%
% Revision 1.3  2004/08/19 06:30:52  roboos
% added extra consistency check for cfg.xgrid etc.
%
% Revision 1.2  2004/08/16 09:32:46  roboos
% added xgrid, ygrid, zgrid and dim to the output grid for consistency
%
% Revision 1.1  2004/06/28 08:59:38  roboos
% moved files from fieldtrip to fieldtrip/private
%
% Revision 1.2  2004/05/07 08:34:30  roberto
% added check for sensors being gradiometers and multisphere volume
%
% Revision 1.1  2004/03/01 11:00:04  roberto
% first entry in CVS, added precomputed filter to grid
%

% set the defaults
if ~isfield(cfg, 'xgrid'),            cfg.xgrid = 'auto';         end
if ~isfield(cfg, 'ygrid'),            cfg.ygrid = 'auto';         end
if ~isfield(cfg, 'zgrid'),            cfg.zgrid = 'auto';         end
if ~isfield(cfg, 'threshold'),        cfg.threshold = 0.1;        end % relative
if ~isfield(cfg, 'smooth');           cfg.smooth    = 5;          end % in voxels
if ~isfield(cfg, 'sourceunits');      cfg.sourceunits  = 'cm';    end
if ~isfield(cfg, 'mriunits');         cfg.mriunits     = 'mm';    end

% convert the source grid locations into the same units as the mri grid locations
if strcmp(cfg.sourceunits, 'cm') && strcmp(cfg.mriunits, 'cm')
  scale = 1;
elseif strcmp(cfg.sourceunits, 'cm') && strcmp(cfg.mriunits, 'mm')
  scale = 10;
elseif strcmp(cfg.sourceunits, 'mm') && strcmp(cfg.mriunits, 'cm')
  scale = 0.1;
elseif strcmp(cfg.sourceunits, 'mm') && strcmp(cfg.mriunits, 'mm')
  scale = 1;
else
  error('unknown physical dimension of the source or mri (can be mm or cm)\n');
end

% Initially a grid is constructed that spans a box which completely encompasses the sensor array.
% Such a grid is much larger than what is needed to span the brain compartment, therefore it will
% be tightened around the brain compartment at the end.
maketightgrid = 0;

if ~isfield(cfg, 'grid') && ~isfield(cfg, 'mri')
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  % construct a regular 3D grid that spans a box encompassing all electrode
  % or gradiometer coils, this will typically also cover the complete brain
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  if (strcmp(cfg.xgrid, 'auto') || strcmp(cfg.ygrid, 'auto') || strcmp(cfg.zgrid, 'auto')) && ~isfield(cfg, 'resolution')
    error('you must specify cfg.resolution in combination with grid=''auto''');
  end
  if ischar(cfg.xgrid) && strcmp(cfg.xgrid, 'auto')
    cfg.xgrid = floor(min(sens.pnt(:,1))):cfg.resolution:ceil(max(sens.pnt(:,1)));
    maketightgrid = 1;
  end
  if ischar(cfg.ygrid) && strcmp(cfg.ygrid, 'auto')
    cfg.ygrid = floor(min(sens.pnt(:,2))):cfg.resolution:ceil(max(sens.pnt(:,2)));
    maketightgrid = 1;
  end
  if ischar(cfg.zgrid) && strcmp(cfg.zgrid, 'auto')
    cfg.zgrid = floor(min(sens.pnt(:,3))):cfg.resolution:ceil(max(sens.pnt(:,3)));
    maketightgrid = 1;
  end
  grid.xgrid = cfg.xgrid;
  grid.ygrid = cfg.ygrid;
  grid.zgrid = cfg.zgrid;
  grid.dim   = [length(grid.xgrid) length(grid.ygrid) length(grid.zgrid)];

elseif ~isfield(cfg, 'grid') && isfield(cfg, 'mri')
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  % construct a grid based on the segmented MRI that is provided in the
  % configuration, only voxels in gray matter will be used
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  if ~isfield(cfg, 'resolution')
    error('you must specify cfg.resolution in combination with a segmented MRI');
  end
  if isstr(cfg.mri)
    % read the segmentation from file
    mri      = read_fcdc_mri(cfg.mri);
    mri.gray = double(mri.anatomy);
  elseif isstruct(cfg.mri) && ~isfield(cfg.mri, 'gray')
    % looks like a segmentation that has already been loaded in memory
    mri      = cfg.mri;
    mri.gray = double(mri.anatomy);
  elseif isstruct(cfg.mri) && isfield(cfg.mri, 'gray')
    % looks like a complete segmentation from SEGMENTVOLUME
    mri      = cfg.mri;
    mri.gray = double(mri.gray);
  else
    error('cannot determine the format of the segmentation in cfg.mri');
  end

  % for compatibility with old MRI volumes
  mri = fixvolume(mri);

  % apply a smoothing of a certain amount of voxels
  if ~isstr(cfg.smooth) && cfg.smooth>1
    fprintf('smoothing gray matter segmentation with %d voxels\n', cfg.smooth);
    mri.gray = spm_conv(mri.gray, cfg.smooth);
  end

  % determine for each voxel whether it belongs to the cortex
  if isfield(cfg, 'threshold')
    fprintf('thresholding gray matter segmentation at a relative value of %f\n', cfg.threshold);
    head = mri.gray./max(mri.gray(:)) > cfg.threshold;
  else
    error('you must specify cfg.threshold for cortex segmentation');
  end

  ind                 = find(head(:));
  fprintf('%d from %d voxels in the segmentation are marked as cortex (%.0f%%)\n', length(ind), prod(size(head)), 100*length(ind)/prod(size(head)));
  [X,Y,Z]             = ndgrid(1:mri.dim(1), 1:mri.dim(2), 1:mri.dim(3));   % create the grid in MRI-coordinates
  posmri              = [X(ind) Y(ind) Z(ind) ones(length(ind),1)];         % take only the inside voxels
  poshead             = mri.transform * posmri';                            % transform to head coordinates
  poshead             = poshead(1:3,:)';
  posmri              = posmri(:,1:3);
  resolution          = cfg.resolution*scale;                                        % source and mri can be expressed in different units (e.g. cm and mm)
  xgrid               = floor(min(poshead(:,1))):resolution:ceil(max(poshead(:,1))); % create the grid in head-coordinates
  ygrid               = floor(min(poshead(:,2))):resolution:ceil(max(poshead(:,2))); % with 'consistent' x,y,z definitions
  zgrid               = floor(min(poshead(:,3))):resolution:ceil(max(poshead(:,3)));
  [X,Y,Z]             = ndgrid(xgrid,ygrid,zgrid);
  pos2head            = [X(:) Y(:) Z(:) ones(length(X(:)),1)]';
  pos2mri             = mri.transform \ pos2head;                           % transform to MRI-coordinates
  pos2mri             = round(pos2mri(1:3,:))';
  pos2head            = pos2head(1:3,:)';
  pos2mri             = pos2mri(:,1:3);
  % it might be that the box with the points does not completely fit into the MRI
  sel = find(pos2mri(:,1)<1 |  pos2mri(:,1)>size(head,1) | ...
             pos2mri(:,2)<1 |  pos2mri(:,2)>size(head,2) | ...
             pos2mri(:,3)<1 |  pos2mri(:,3)>size(head,3)); 
  if isempty(sel)
    % use the efficient implementation
    inside = head(sub2ind(mri.dim, pos2mri(:,1), pos2mri(:,2), pos2mri(:,3)));
  else
    % only loop over the points that can be dealt with
    inside = zeros(length(xgrid)*length(ygrid)*length(zgrid), 1);
    for i=setdiff(1:size(pos2mri,1), sel(:)')
      inside(i) = head(pos2mri(i,1), pos2mri(i,2), pos2mri(i,3));
    end
  end
  inside = find(inside);

  grid.pos            = pos2head/scale;                                     % convert to source units
  grid.xgrid          = xgrid/scale;                                        % convert to source units
  grid.ygrid          = ygrid/scale;                                        % convert to source units
  grid.zgrid          = zgrid/scale;                                        % convert to source units
  grid.dim            = [length(grid.xgrid) length(grid.ygrid) length(grid.zgrid)];
  grid.inside         = inside(:);
  grid.outside        = setdiff(1:size(grid.pos,1),grid.inside)';

  fprintf('the regular 3D grid encompassing the cortex contains %d grid points\n', size(grid.pos,1));
  fprintf('%d grid points inside gray matter\n', length(grid.inside));
  fprintf('%d grid points outside gray matter\n', length(grid.outside));

else
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % a grid is already specified, reuse as much of the prespecified grid
  % as possible (but only known objects)
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if isfield(cfg.grid, 'pos')
    grid.pos = cfg.grid.pos;
  end
  if isfield(cfg.grid, 'dim')
    grid.dim = cfg.grid.dim;
  end
  if isfield(cfg.grid, 'xgrid')
    grid.xgrid = cfg.grid.xgrid;
    cfg.xgrid = grid.xgrid;	% for later reference
  end
  if isfield(cfg.grid, 'ygrid')
    grid.ygrid = cfg.grid.ygrid;
    cfg.ygrid = grid.ygrid;	% for later reference
  end
  if isfield(cfg.grid, 'zgrid')
    grid.zgrid = cfg.grid.zgrid;
    cfg.zgrid = grid.zgrid;	% for later reference
  end
  if isfield(cfg.grid, 'inside')
    grid.inside = cfg.grid.inside;
  end
  if isfield(cfg.grid, 'outside')
    grid.outside = cfg.grid.outside;
  end
  if isfield(cfg.grid, 'lbex')
    grid.lbex = cfg.grid.lbex;
  end
  if isfield(cfg.grid, 'subspace')
    grid.subspace = cfg.grid.subspace;
  end
  if isfield(cfg.grid, 'leadfield')
    grid.leadfield = cfg.grid.leadfield;
  end
  if isfield(cfg.grid, 'filter')
    grid.filter = cfg.grid.filter;
  elseif isfield(cfg.grid, 'avg')
    % the precomputed filters could also be in the average sub-structure
    if isfield(cfg.grid.avg, 'filter')
      grid.filter = cfg.grid.avg.filter;
    end
  end
  % this will remove all remaining unknown objects from the configuration
  cfg.grid = grid;
end

if ~isfield(grid, 'pos')
  % create the regular grid that will be scanned with a dipole
  [X, Y, Z] = ndgrid(cfg.xgrid, cfg.ygrid, cfg.zgrid);
  grid.pos   = [X(:) Y(:) Z(:)];
  grid.xgrid = cfg.xgrid;
  grid.ygrid = cfg.ygrid;
  grid.zgrid = cfg.zgrid;
end

% determine the dipole locations inside the brain volume
if ~isfield(grid, 'inside') && ~isfield(grid, 'outside')
  if isempty(vol)
    % an empty vol in combination with gradiometers indicates a magnetic dipole
    % in an infinite vacuum, i.e. all dipoles can be considered to be inside
    grid.inside = 1:size(grid.pos,1);
    grid.outside = [];
  else
    if isfield(sens, 'ori') && isfield(sens, 'pnt') && isfield(sens, 'tra')
      % in case of MEG, make a triangulation of the skin surface
      [pnt, tri] = headsurface(vol, sens, 'surface', 'skin');
    else
      % in case of EEG, make a triangulation of the brain surface
      [pnt, tri] = headsurface(vol, sens, 'surface', 'brain');
    end
    % determine the dipole positions that are inside the triangulated surface
    tmp = bounding_mesh(grid.pos, pnt, tri);
    grid.inside  = find(tmp==1);
    grid.outside = find(tmp==0);
  end
elseif ~isfield(grid, 'inside')
  grid.inside = setdiff(1:size(grid.pos,1), grid.outside);
elseif ~isfield(grid, 'outside')
  grid.outside = setdiff(1:size(grid.pos,1), grid.inside);
end

if maketightgrid
  fprintf('%d dipoles inside, %d dipoles outside brain\n', length(grid.inside), length(grid.outside));
  fprintf('making tight grid\n');
  xmin = min(grid.pos(grid.inside,1));
  ymin = min(grid.pos(grid.inside,2));
  zmin = min(grid.pos(grid.inside,3));
  xmax = max(grid.pos(grid.inside,1));
  ymax = max(grid.pos(grid.inside,2));
  zmax = max(grid.pos(grid.inside,3));
  xmin_indx = find(grid.xgrid==xmin); 
  ymin_indx = find(grid.ygrid==ymin); 
  zmin_indx = find(grid.zgrid==zmin); 
  xmax_indx = find(grid.xgrid==xmax); 
  ymax_indx = find(grid.ygrid==ymax); 
  zmax_indx = find(grid.zgrid==zmax); 
  sel =       (grid.pos(:,1)>=xmin & grid.pos(:,1)<=xmax); % select all grid positions inside the tight box
  sel = sel & (grid.pos(:,2)>=ymin & grid.pos(:,2)<=ymax); % select all grid positions inside the tight box
  sel = sel & (grid.pos(:,3)>=zmin & grid.pos(:,3)<=zmax); % select all grid positions inside the tight box
  grid.pos   = grid.pos(sel,:);
  % update the inside and outside vector
  tmp = zeros(1,prod(grid.dim));
  tmp(grid.inside)  = 1;        % these are originally inside the brain
  tmp(grid.outside) = 0;        % these are originally outside the brain
  tmp               = tmp(sel); % within the tight box, these are inside the brain
  grid.inside  = find(tmp);
  grid.outside = find(~tmp);
  grid.xgrid   = grid.xgrid(xmin_indx:xmax_indx);
  grid.ygrid   = grid.ygrid(ymin_indx:ymax_indx);
  grid.zgrid   = grid.zgrid(zmin_indx:zmax_indx);
  grid.dim     = [length(grid.xgrid) length(grid.ygrid) length(grid.zgrid)];
end

fprintf('%d dipoles inside, %d dipoles outside brain\n', length(grid.inside), length(grid.outside));

